from dataclasses import dataclass
from typing import List

from dubo.api_client.models import QueryStatus


@dataclass
class DataResult:
    id: str
    query_text: str
    status: QueryStatus
    results_set: List[dict]
    row_count: int

    def validate(self) -> bool:
        if self.status not in [
            "running",
            "completed",
            "failed",
        ]:
            return False
        return True



import time
from typing import Union

from dubo.api_client import AuthenticatedClient, Client
from dubo.api_client.api.enterprise import (
    ask_dispatch_api_v1_dubo_query_generate_post,
    ask_poll_api_v1_dubo_query_retrieve_get,
)
from dubo.api_client.models import *
from dubo.common import DuboException

from dubo.config import get_dubo_key, RETRIEVE_RESULT_MAX_DELAY
from dubo.entities import DataResult


def dispatch_query(
    client: Union[AuthenticatedClient, Client], query: str, fast: bool = False
) -> str:
    """
    Dispatch the query and get a tracking_id.
    """
    api_key = get_dubo_key()
    json_body = CreateApiQuery(
        query_text=query,
        fast=fast,
    )

    res = ask_dispatch_api_v1_dubo_query_generate_post.sync(
        client=client,
        x_dubo_key=api_key,
        json_body=json_body,
    )
    if not res or not res.id:
        raise DuboException("Failed to dispatch query.")
    return res.id


def retrieve_result(
    client: Union[AuthenticatedClient, Client], tracking_id: str
) -> DataResult:
    """
    Poll for the result using the provided tracking_id.
    """
    delay = 0.1
    max_delay = RETRIEVE_RESULT_MAX_DELAY
    api_key = get_dubo_key()
    while True:
        res = ask_poll_api_v1_dubo_query_retrieve_get.sync(
            client=client,
            x_dubo_key=api_key,
            dispatch_id=tracking_id,
        )
        if res.status == QueryStatus.SUCCESS:
            return DataResult(
                id=res.id,
                query_text=res.query_text,
                status=res.status,
                results_set=[item.additional_properties for item in res.results_set],
                row_count=res.row_count,
            )
        elif res.status == QueryStatus.FAILED:
            raise DuboException(res["error"])
        else:
            time.sleep(delay)
            delay = min(delay * 2, max_delay)


def dispatch_and_retrieve(
    client: Union[AuthenticatedClient, Client], query: str, fast: bool = False
) -> DataResult:
    """
    Convenience function to generate the query and retrieve the result.
    """
    tracking_id = dispatch_query(client, query, fast)
    return retrieve_result(client, tracking_id)



import json
import os
from http import HTTPStatus
from uuid import UUID
import sqlglot

import sqlite3
from typing import Dict, List, Optional, Type

import pandas as pd
import altair as alt
from pydeck.io.html import deck_to_html

from dubo.common import DuboException
from dubo.config import BASE_API_URL, get_dubo_key
from dubo.entities import DataResult
from dubo.query_utils import dispatch_and_retrieve

from dubo.api_client import Client as DuboApiClient
from dubo.api_client.api.dubo import (
    read_query_v1_dubo_query_get,
)

from dubo.api_client.api.enterprise import (
    ask_dispatch_api_v1_dubo_query_generate_post,
    create_documentation_api_v1_dubo_documentation_post,
    delete_document_by_id_api_v1_dubo_documentation_delete,
    filter_documentation_endpoint_api_v1_dubo_query_filter_documentation_get,
    read_all_api_v1_dubo_documentation_get,
    read_one_api_v1_dubo_documentation_data_source_documentation_id_get,
    update_document_api_v1_dubo_documentation_put,
)
from dubo.api_client.api.sdk import (
    create_dubo_chart_v1_dubo_chart_post,
    get_query_execution_category_v1_dubo_categorize_chart_get,
)
from dubo.api_client.models import *
from dubo.api_client.types import *
from dubo.api_client.models.matched_doc import MatchedDoc


client = DuboApiClient(base_url=BASE_API_URL)  # type: ignore


def ask(
    query: str,
    data: List[pd.DataFrame] | pd.DataFrame,
    verbose: bool = True,
    rtype: Type = pd.DataFrame,
    column_descriptions: Optional[Dict[str, str]] = None,
) -> pd.DataFrame | List:
    """
    Ask Dubo a question about your data.

    :param query: The question to ask Dubo.
    :param data: The DataFrame to ask Dubo about.
    :param verbose: Whether to print the query that Dubo is running.
    :param rtype: Expected returned type.
    :param column_descriptions: A dictionary of column names to descriptions.
    :type query: str
    :type rtype: pd.DataFrame or list
    :return: The result of the query.

    ##### Example
    ```python
    import pandas as pd
    from dubo import ask

    data = pd.DataFrame({'a': [1, 2, 3], 'b': [4, 5, 6]})
    ask('What is the sum of a?', data, rtype=list)

    # [(6,)]
    ```
    """
    conn = sqlite3.connect(":memory:")

    if isinstance(data, pd.DataFrame):
        data = [data]
    if not all([isinstance(d, pd.DataFrame) for d in data]):
        raise TypeError(
            "Input for data must be a pandas.DataFrame but saw type: %s" % type(data)
        )
    schemas = []
    for i, dset in enumerate(data):
        tbl_name = f"tbl_{i}"
        dset.infer_objects().to_sql(tbl_name, conn, index=False)
        schema = conn.execute(
            f"SELECT sql FROM sqlite_schema WHERE name = '{tbl_name}'"
        ).fetchone()
        schemas.append(schema[0])
    possible_query = read_query_v1_dubo_query_get.sync(
        client=client,
        user_query=query,
        schemas=schemas,
        descriptions=column_descriptions,
        json_body=BodyReadQueryV1DuboQueryGet(),
    )

    if not possible_query:
        raise DuboException(f"Unable to produce a result for the query: {query}")
    try:
        result = possible_query.query_text
        if verbose:
            print(result)
    except KeyError:
        raise DuboException(f"Unable to produce a result for the query: {query}")
    try:
        if rtype == pd.DataFrame:
            return pd.read_sql(result, conn)
        elif rtype == list:
            return conn.execute(result).fetchall()
        else:
            raise TypeError(
                f"rtype must be either pd.DataFrame or list but saw type: {rtype}"
            )
    except Exception as e:
        raise DuboException(str(e))


def chart(
    query: str,
    df: pd.DataFrame,
    chart_type: ChartType | None = None,
    verbose=False,
    **kwargs,
):
    """
    Ask Dubo to generate a chart.

    :param query: The chart to ask Dubo to generate.
    :param df: The DataFrame for the chart.
    :param chart_type: Type of chart: ChartType.DECK_GL or ChartType.VEGA_LITE.
    :param verbose: Whether to print verbose logs.
    :type query: str
    :type df: pd.DataFrame
    :type chart_type: ChartType | None
    :type verbose: bool
    :return: The chart.

    ##### Example
    ```python
    import pandas as pd

    from dubo import chart
    from dubo.api_client.models import ChartType

    housing_df = pd.read_csv("https://raw.githubusercontent.com/ajduberstein/geo_datasets/master/housing.csv")

    chart(
        query="Map the houses",
        df=housing_df,
        chart_type=ChartType.DECK_GL,
        as_string=True,
    )

    # <!DOCTYPE html>
    #    <html>
    #    ...
    #    <body>
    #        <div id="deck-container">
    #        </div>
    #    </body>
    #    <script>
    #        const container = document.getElementById('deck-container');
    #        ...
    ```
    """
    if not chart_type:
        chart_type_str = get_query_execution_category_v1_dubo_categorize_chart_get.sync(
            client=client,
            text_input=query,
        )
        chart_type = ChartType(str(chart_type_str).lower())
    else:
        chart_type = ChartType(str(chart_type))
    if ChartType(chart_type) not in (
        ChartType.VEGA_LITE,
        ChartType.DECK_GL,
    ):
        raise ValueError(
            f"Chart type must be one of: VEGA_LITE, DECK_GL, got {chart_type}"
        )

    if verbose:
        print("Generating a chart of type:", chart_type)

    data_snippet = df.head().to_dict(orient="records")
    data_snippet = [
        DuboChartQueryDataSnippetItem.from_dict(item) for item in data_snippet
    ]

    charts = create_dubo_chart_v1_dubo_chart_post.sync(
        client=client,
        json_body=DuboChartQuery(
            user_query=query,
            data_snippet=data_snippet,
            fast=False,
            chart_type=chart_type,
        ),
    )

    if chart_type == ChartType.VEGA_LITE:
        chart = charts[0]
        chart["data"] = {"values": df.sample(10000).to_dict(orient="records")}
        chart["height"] = kwargs.get("height") or 390
        chart["width"] = kwargs.get("width") or 500
        return alt.Chart.from_dict(chart, **kwargs)

    if chart_type == ChartType.DECK_GL:
        chart = charts[0]
        for layer in chart["layers"]:
            if "data" in layer:
                layer["data"] = df.to_dict(orient="records")
        mapbox_key = (
            kwargs.pop("mapbox_key", None)
            or os.environ.get("MAPBOX_KEY")
            or ""  # noqa: E501
        )
        return deck_to_html(json.dumps(chart), mapbox_key=mapbox_key, **kwargs)

    raise ValueError(f"Unknown chart type: {chart_type}")


def query(
    query_text: str,
    fast: bool = False,
) -> DataResult:
    """
    Ask Dubo a question.

    :param query_text: The question to ask Dubo.
    :param fast: Use faster, less accurate model
    :type query_text: str
    :type fast: bool
    :return: The SQL query.

    ##### Example
    ```python
    from dubo import query

    query("How many area types are there?")

    # DataResult(
    #  id='query-56513883-6749-4f2b-ad57-7bb8cb350161',
    #  query_text='How many area types are there?',
    #  status=QueryStatus.SUCCESS,
    #  results_set=[{'count': 9}],
    #  row_count=1
    # )
    ```
    """
    get_dubo_key()
    return dispatch_and_retrieve(client, query_text, fast)


def generate_sql(
    query_text: str,
    fast: bool = False,
    pretty: bool = True,
) -> str:
    """
    Ask Dubo to generate a SQL query.

    :param query_text: The plain text query.
    :param fast: Use faster, less accurate model
    :type query_text: str
    :type fast: bool
    :return: The SQL query.

    ##### Example
    ```python
    from dubo import query

    query("How many area types are there?")

    # "SELECT COUNT(DISTINCT type) AS num_area_types FROM public.area"
    ```
    """
    api_key = get_dubo_key()
    body = CreateApiQuery(
        query_text=query_text,
        fast=fast,
        mode=CreateApiQueryMode.JUST_SQL_TEXT,
    )
    res = ask_dispatch_api_v1_dubo_query_generate_post.sync(
        client=client,
        x_dubo_key=api_key,
        json_body=body,
    )
    if not res:
        raise DuboException("Failed to generate SQL.")
    if pretty and res.sql_text:
        return sqlglot.parse_one(res.sql_text).sql(pretty=True)
    return res.sql_text


def search_tables(
    query_text: str,
    fast: bool = False,
) -> List[AttenuatedDDL]:
    """
    Ask Dubo to return the list of tables that are a potential match for this query.

    :param query_text: The plain text query.
    :param fast: Use faster, less accurate model
    :type query_text: str
    :type fast: bool
    :return: The list of tables.

    ##### Example
    ```python
    from dubo import query

    query("How many area types are there?")

    # [
    #   AttenuatedDDL(
    #      cols=[
    #          TableColumn(column_name='name', data_type='character varying', is_nullable=False, table_name='area_type', schema_name='public', is_partitioning_column=None),
    #          TableColumn(column_name='parent', data_type='integer', is_nullable=True, table_name='area_type', schema_name='public', is_partitioning_column=None),
    #          TableColumn(column_name='description', data_type='text', is_nullable=True, table_name='area_type', schema_name='public', is_partitioning_column=None),
    #          TableColumn(column_name='gid', data_type='uuid', is_nullable=False, table_name='area_type', schema_name='public', is_partitioning_column=None),
    #          TableColumn(column_name='id', data_type='integer', is_nullable=False, table_name='area_type', schema_name='public', is_partitioning_column=None),
    #          TableColumn(column_name='child_order', data_type='integer', is_nullable=False, table_name='area_type', schema_name='public', is_partitioning_column=None),
    #      ],
    #      table_name='area_type',
    #      schema_name='public',
    #      id='e3e10474-aa55-4654-bfb8-6592dc540127',
    #      database_name='postgres',
    #      description='Table storing different types of areas, including their parent-child relationships and descriptions',
    #   ),
    #  ...
    # ]
    ```
    """
    api_key = get_dubo_key()
    body = CreateApiQuery(
        query_text=query_text,
        fast=fast,
        mode=CreateApiQueryMode.JUST_TABLES,
    )
    res = ask_dispatch_api_v1_dubo_query_generate_post.sync(
        client=client,
        x_dubo_key=api_key,
        json_body=body,
    )
    return res.tables


def filter_documentation(
    user_query: str,
    data_source_documentation_id: Optional[str] = None,
    page_number: int = 1,
    page_size: int = 25,
) -> List[MatchedDoc]:
    """
    Search in documentation

    :param user_query: The search query.
    :param data_source_documentation_id: The documentation id.
    :param page_number: The page number.
    :param page_size: The page size.
    :type user_query: str
    :type data_source_documentation_id: str, optional
    :type page_number: int, optional
    :type page_size: int, optional
    :return: The list of results (paginated): body, score and matched_doc_id (datasource id)

    ##### Example
    ```python
    from dubo import filter_documentation

    res = filter_documentation(
        user_query="describe the area type table",
        data_source_documentation_id="c1d62c33-4561-4b5f-b2c2-e0203cee1f7b",
        page_number=1,
        page_size=10,
    )
    # [ MatchedDoc(
    #    body='The area_type table contains the list of area types',
    #    score=0.947,
    #    matched_doc_id='2be09019-8d48-49a0-aeb0-4837a57e444c'
    #  ), ...]
    ```
    """
    api_key = get_dubo_key()

    res = filter_documentation_endpoint_api_v1_dubo_query_filter_documentation_get.sync(
        client=client,
        x_dubo_key=api_key,
        user_query=user_query,
        data_source_documentation_id=data_source_documentation_id,
        page_number=page_number,
        page_size=page_size,
    )

    return res.data


def create_doc(
    file_path: str,
    shingle_length: int = 1000,
    step: int = 500,
) -> DataSourceDocument:
    """
    Create Documentation.

    :param file_path: The path to the file to upload.
    :param shingle_length: TBC.
    :param step: TBC.
    :return: The documentation

    ##### Example
    ```python
    from dubo import create_doc

    create_doc(
        file_path="./documentation.txt",
        shingle_length=1000,
        step=500,
    )

    # DataSourceDocument(
    #   id='c1d62c33-4561-4b5f-b2c2-e0203cee1f7b',
    #   file_name='documentation.txt',
    #   data_source_id=...,
    #   organization_id=...,
    #   created_at=...,
    #   updated_at=...,
    # )
    ```
    """
    api_key = get_dubo_key()

    with open(file_path, "rb") as doc:
        file_name = os.path.basename(file_path)
        file = File(
            payload=doc,
            file_name=file_name,
        )

        body = BodyCreateDocumentationApiV1DuboDocumentationPost(file)

        res = create_documentation_api_v1_dubo_documentation_post.sync_detailed(
            client=client,
            x_dubo_key=api_key,
            multipart_data=body,
            shingle_length=shingle_length,
            step=step,
        )

        if res.status_code == HTTPStatus.OK:
            return res.parsed
        else:
            raise DuboException(
                f"Documentation create failed with status code {res.status_code}: {res.content.decode('utf-8')}"
            )


def get_doc(data_source_documentation_id: str) -> DataSourceDocument:
    """
    Get one document by ID.

    :param data_source_documentation_id: The ID of the document to get.
    :return: The document

    ##### Example
    ```python
    from dubo import get_doc

    get_doc("c1d62c33-4561-4b5f-b2c2-e0203cee1f7b")

    # DataSourceDocument(
    #   id='c1d62c33-4561-4b5f-b2c2-e0203cee1f7b',
    #   file_name='documentation.txt',
    #   data_source_id=...,
    #   organization_id=...,
    #   created_at=...,
    #   updated_at=...,
    # )
    ```
    """
    api_key = get_dubo_key()
    res = read_one_api_v1_dubo_documentation_data_source_documentation_id_get.sync_detailed(
        client=client,
        data_source_documentation_id=data_source_documentation_id,
        x_dubo_key=api_key,
    )

    if res.status_code == HTTPStatus.OK:
        return res.parsed
    else:
        raise DuboException(
            f"Documentation get failed with status code {res.status_code}: {res.content.decode('utf-8')}"
        )


def get_all_docs() -> List[Dict[str, str]]:
    """
    Get All Documents.

    :return: The list of documents (file_name and id)

    ##### Example
    ```python
    from dubo import get_all_docs

    get_all_docs()

    # [{'file_name': 'documentation.txt', 'id': 'c1d62c33-4561-4b5f-b2c2-e0203cee1f7b'}]
    ```
    """
    api_key = get_dubo_key()
    res = read_all_api_v1_dubo_documentation_get.sync(
        client=client,
        x_dubo_key=api_key,
    )

    if res is None:
        return []

    return [{"file_name": doc.file_name, "id": doc.id} for doc in res]


def update_doc(
    data_source_documentation_id: str,
    file_path: str,
    shingle_length: int = 1000,
    step: int = 500,
) -> bool:
    """
    Update Document.

    :param data_source_documentation_id: The ID of the document to update.
    :param file_path: The path to the file to upload.
    :param shingle_length: TBC.
    :param step: TBC.
    :return: True if successful, False otherwise

    ##### Example
    ```python
    from dubo import update_doc

    update_doc(
        data_source_documentation_id="c1d62c33-4561-4b5f-b2c2-e0203cee1f7b",
        file_path="./documentation.txt",
        shingle_length=1000,
        step=500,
    )

    # True
    ```
    """
    api_key = get_dubo_key()

    with open(file_path, "rb") as doc:
        file_name = os.path.basename(file_path)
        file = File(
            payload=doc,
            file_name=file_name,
        )
        body = BodyUpdateDocumentApiV1DuboDocumentationPut(
            file=file,
        )
        res = update_document_api_v1_dubo_documentation_put.sync_detailed(
            client=client,
            x_dubo_key=api_key,
            data_source_documentation_id=str(UUID(data_source_documentation_id)),
            shingle_length=shingle_length,
            step=step,
            multipart_data=body,
        )

        if res.status_code == HTTPStatus.OK:
            return True
        else:
            raise DuboException(
                f"Documentation update failed with status code {res.status_code}: {res.content.decode('utf-8')}"
            )


def delete_doc(data_source_documentation_id: str) -> bool:
    """
    Delete a document by its ID.

    :param data_source_documentation_id: The ID of the document to delete.
    :return: True if the deletion was successful, False otherwise

    ##### Example
    ```python
    from dubo import delete_doc

    delete_doc("c1d62c33-4561-4b5f-b2c2-e0203cee1f7b")

    # True
    ```
    """
    # No need to fetch by name, just use the provided ID directly.
    api_key = get_dubo_key()

    res = delete_document_by_id_api_v1_dubo_documentation_delete.sync(
        client=client,
        x_dubo_key=api_key,
        data_source_documentation_id=data_source_documentation_id,
    )

    return res



""" Contains all the data models used in inputs/outputs """

from .ask_dispatch_response import AskDispatchResponse
from .ask_dispatch_response_results_set_item import AskDispatchResponseResultsSetItem
from .attenuated_ddl import AttenuatedDDL
from .body_create_documentation_api_v1_dubo_documentation_post import BodyCreateDocumentationApiV1DuboDocumentationPost
from .body_read_query_v1_dubo_query_get import BodyReadQueryV1DuboQueryGet
from .body_update_document_api_v1_dubo_documentation_put import BodyUpdateDocumentApiV1DuboDocumentationPut
from .chart_type import ChartType
from .create_api_query import CreateApiQuery
from .create_api_query_mode import CreateApiQueryMode
from .data_source_document import DataSourceDocument
from .dubo_chart_query import DuboChartQuery
from .dubo_chart_query_chart_spec import DuboChartQueryChartSpec
from .dubo_chart_query_data_snippet_item import DuboChartQueryDataSnippetItem
from .dubo_example import DuboExample
from .http_validation_error import HTTPValidationError
from .matched_doc import MatchedDoc
from .page_matched_doc import PageMatchedDoc
from .query_response import QueryResponse
from .query_status import QueryStatus
from .table_column import TableColumn
from .validation_error import ValidationError

__all__ = (
    "AskDispatchResponse",
    "AskDispatchResponseResultsSetItem",
    "AttenuatedDDL",
    "BodyCreateDocumentationApiV1DuboDocumentationPost",
    "BodyReadQueryV1DuboQueryGet",
    "BodyUpdateDocumentApiV1DuboDocumentationPut",
    "ChartType",
    "CreateApiQuery",
    "CreateApiQueryMode",
    "DataSourceDocument",
    "DuboChartQuery",
    "DuboChartQueryChartSpec",
    "DuboChartQueryDataSnippetItem",
    "DuboExample",
    "HTTPValidationError",
    "MatchedDoc",
    "PageMatchedDoc",
    "QueryResponse",
    "QueryStatus",
    "TableColumn",
    "ValidationError",
)



from .ask_dubo import *  # noqa
from .config import DOCS_URL, FEEDBACK_URL


__title__ = "dubo"
__description__ = "Natural language analytics on any data frame or database."
__version__ = "0.2.8"


MSG = f"Dubo | Examples: {DOCS_URL} | Contact: {FEEDBACK_URL} | Privacy policy: https://mercator.tech/privacy"

print(MSG)



from typing import Any, Dict, List, Type, TypeVar, Union

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..models.create_api_query_mode import CreateApiQueryMode
from ..types import UNSET, Unset

T = TypeVar("T", bound="CreateApiQuery")


@_attrs_define
class CreateApiQuery:
    """Developer API query POST body. NOTE: DO NOT use this in other parts of the application. Prefer QueryExecutionCreate.

    Attributes:
        query_text (str):
        fast (Union[Unset, bool]):
        mode (Union[Unset, CreateApiQueryMode]): Developer API query POST body. NOTE: DO NOT use this in other parts of
            the application. Prefer QueryExecutionCreate. Default: CreateApiQueryMode.FULL_EXECUTION.
    """

    query_text: str
    fast: Union[Unset, bool] = False
    mode: Union[Unset, CreateApiQueryMode] = CreateApiQueryMode.FULL_EXECUTION
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        query_text = self.query_text
        fast = self.fast
        mode: Union[Unset, str] = UNSET
        if not isinstance(self.mode, Unset):
            mode = self.mode.value

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "query_text": query_text,
            }
        )
        if fast is not UNSET:
            field_dict["fast"] = fast
        if mode is not UNSET:
            field_dict["mode"] = mode

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        query_text = d.pop("query_text")

        fast = d.pop("fast", UNSET)

        _mode = d.pop("mode", UNSET)
        mode: Union[Unset, CreateApiQueryMode]
        if isinstance(_mode, Unset):
            mode = UNSET
        else:
            mode = CreateApiQueryMode(_mode)

        create_api_query = cls(
            query_text=query_text,
            fast=fast,
            mode=mode,
        )

        create_api_query.additional_properties = d
        return create_api_query

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import TYPE_CHECKING, Any, Dict, List, Type, TypeVar, Union, cast

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.table_column import TableColumn


T = TypeVar("T", bound="AttenuatedDDL")


@_attrs_define
class AttenuatedDDL:
    """
    Attributes:
        cols (List['TableColumn']):
        table_name (str):
        schema_name (str):
        id (Union[Unset, str]):
        partition_columns (Union[Unset, List[str]]):
        database_name (Union[Unset, str]):
        description (Union[Unset, str]):
        foreign_keys (Union[Unset, List[str]]):
        page_rank (Union[Unset, int]):
    """

    cols: List["TableColumn"]
    table_name: str
    schema_name: str
    id: Union[Unset, str] = UNSET
    partition_columns: Union[Unset, List[str]] = UNSET
    database_name: Union[Unset, str] = UNSET
    description: Union[Unset, str] = UNSET
    foreign_keys: Union[Unset, List[str]] = UNSET
    page_rank: Union[Unset, int] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        cols = []
        for cols_item_data in self.cols:
            cols_item = cols_item_data.to_dict()

            cols.append(cols_item)

        table_name = self.table_name
        schema_name = self.schema_name
        id = self.id
        partition_columns: Union[Unset, List[str]] = UNSET
        if not isinstance(self.partition_columns, Unset):
            partition_columns = self.partition_columns

        database_name = self.database_name
        description = self.description
        foreign_keys: Union[Unset, List[str]] = UNSET
        if not isinstance(self.foreign_keys, Unset):
            foreign_keys = self.foreign_keys

        page_rank = self.page_rank

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "cols": cols,
                "table_name": table_name,
                "schema_name": schema_name,
            }
        )
        if id is not UNSET:
            field_dict["id"] = id
        if partition_columns is not UNSET:
            field_dict["partition_columns"] = partition_columns
        if database_name is not UNSET:
            field_dict["database_name"] = database_name
        if description is not UNSET:
            field_dict["description"] = description
        if foreign_keys is not UNSET:
            field_dict["foreign_keys"] = foreign_keys
        if page_rank is not UNSET:
            field_dict["page_rank"] = page_rank

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.table_column import TableColumn

        d = src_dict.copy()
        cols = []
        _cols = d.pop("cols")
        for cols_item_data in _cols:
            cols_item = TableColumn.from_dict(cols_item_data)

            cols.append(cols_item)

        table_name = d.pop("table_name")

        schema_name = d.pop("schema_name")

        id = d.pop("id", UNSET)

        partition_columns = cast(List[str], d.pop("partition_columns", UNSET))

        database_name = d.pop("database_name", UNSET)

        description = d.pop("description", UNSET)

        foreign_keys = cast(List[str], d.pop("foreign_keys", UNSET))

        page_rank = d.pop("page_rank", UNSET)

        attenuated_ddl = cls(
            cols=cols,
            table_name=table_name,
            schema_name=schema_name,
            id=id,
            partition_columns=partition_columns,
            database_name=database_name,
            description=description,
            foreign_keys=foreign_keys,
            page_rank=page_rank,
        )

        attenuated_ddl.additional_properties = d
        return attenuated_ddl

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from enum import Enum


class CreateApiQueryMode(str, Enum):
    FULL_EXECUTION = "full_execution"
    JUST_SQL_TEXT = "just_sql_text"
    JUST_TABLES = "just_tables"

    def __str__(self) -> str:
        return str(self.value)



from typing import Any, Dict, List, Type, TypeVar, Union

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

T = TypeVar("T", bound="TableColumn")


@_attrs_define
class TableColumn:
    """
    Attributes:
        column_name (str):
        data_type (str):
        is_nullable (bool):
        table_name (str):
        schema_name (str):
        is_partitioning_column (Union[Unset, str]):
    """

    column_name: str
    data_type: str
    is_nullable: bool
    table_name: str
    schema_name: str
    is_partitioning_column: Union[Unset, str] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        column_name = self.column_name
        data_type = self.data_type
        is_nullable = self.is_nullable
        table_name = self.table_name
        schema_name = self.schema_name
        is_partitioning_column = self.is_partitioning_column

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "column_name": column_name,
                "data_type": data_type,
                "is_nullable": is_nullable,
                "table_name": table_name,
                "schema_name": schema_name,
            }
        )
        if is_partitioning_column is not UNSET:
            field_dict["is_partitioning_column"] = is_partitioning_column

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        column_name = d.pop("column_name")

        data_type = d.pop("data_type")

        is_nullable = d.pop("is_nullable")

        table_name = d.pop("table_name")

        schema_name = d.pop("schema_name")

        is_partitioning_column = d.pop("is_partitioning_column", UNSET)

        table_column = cls(
            column_name=column_name,
            data_type=data_type,
            is_nullable=is_nullable,
            table_name=table_name,
            schema_name=schema_name,
            is_partitioning_column=is_partitioning_column,
        )

        table_column.additional_properties = d
        return table_column

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



import datetime
from typing import Any, Dict, List, Type, TypeVar

from attrs import define as _attrs_define
from attrs import field as _attrs_field
from dateutil.parser import isoparse

T = TypeVar("T", bound="DataSourceDocument")


@_attrs_define
class DataSourceDocument:
    """
    Attributes:
        id (str):
        created_at (datetime.datetime):
        updated_at (datetime.datetime):
        file_name (str):
        data_source_id (str):
        organization_id (str):
    """

    id: str
    created_at: datetime.datetime
    updated_at: datetime.datetime
    file_name: str
    data_source_id: str
    organization_id: str
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        id = self.id
        created_at = self.created_at.isoformat()

        updated_at = self.updated_at.isoformat()

        file_name = self.file_name
        data_source_id = self.data_source_id
        organization_id = self.organization_id

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "id": id,
                "created_at": created_at,
                "updated_at": updated_at,
                "file_name": file_name,
                "data_source_id": data_source_id,
                "organization_id": organization_id,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        id = d.pop("id")

        created_at = isoparse(d.pop("created_at"))

        updated_at = isoparse(d.pop("updated_at"))

        file_name = d.pop("file_name")

        data_source_id = d.pop("data_source_id")

        organization_id = d.pop("organization_id")

        data_source_document = cls(
            id=id,
            created_at=created_at,
            updated_at=updated_at,
            file_name=file_name,
            data_source_id=data_source_id,
            organization_id=organization_id,
        )

        data_source_document.additional_properties = d
        return data_source_document

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import TYPE_CHECKING, Any, Dict, List, Type, TypeVar, Union

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..models.query_status import QueryStatus
from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.ask_dispatch_response_results_set_item import AskDispatchResponseResultsSetItem
    from ..models.attenuated_ddl import AttenuatedDDL


T = TypeVar("T", bound="AskDispatchResponse")


@_attrs_define
class AskDispatchResponse:
    """Response to a developer API query dispatch request. NOTE: DO NOT use this in other parts of the application. Prefer
    QueryExecution.

        Attributes:
            id (str):
            query_text (str):
            status (QueryStatus): An enumeration.
            tables (Union[Unset, List['AttenuatedDDL']]):
            results_set (Union[Unset, List['AskDispatchResponseResultsSetItem']]):
            sql_text (Union[Unset, str]):
            row_count (Union[Unset, int]):
    """

    id: str
    query_text: str
    status: QueryStatus
    tables: Union[Unset, List["AttenuatedDDL"]] = UNSET
    results_set: Union[Unset, List["AskDispatchResponseResultsSetItem"]] = UNSET
    sql_text: Union[Unset, str] = UNSET
    row_count: Union[Unset, int] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        id = self.id
        query_text = self.query_text
        status = self.status.value

        tables: Union[Unset, List[Dict[str, Any]]] = UNSET
        if not isinstance(self.tables, Unset):
            tables = []
            for tables_item_data in self.tables:
                tables_item = tables_item_data.to_dict()

                tables.append(tables_item)

        results_set: Union[Unset, List[Dict[str, Any]]] = UNSET
        if not isinstance(self.results_set, Unset):
            results_set = []
            for results_set_item_data in self.results_set:
                results_set_item = results_set_item_data.to_dict()

                results_set.append(results_set_item)

        sql_text = self.sql_text
        row_count = self.row_count

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "id": id,
                "query_text": query_text,
                "status": status,
            }
        )
        if tables is not UNSET:
            field_dict["tables"] = tables
        if results_set is not UNSET:
            field_dict["results_set"] = results_set
        if sql_text is not UNSET:
            field_dict["sql_text"] = sql_text
        if row_count is not UNSET:
            field_dict["row_count"] = row_count

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.ask_dispatch_response_results_set_item import AskDispatchResponseResultsSetItem
        from ..models.attenuated_ddl import AttenuatedDDL

        d = src_dict.copy()
        id = d.pop("id")

        query_text = d.pop("query_text")

        status = QueryStatus(d.pop("status"))

        tables = []
        _tables = d.pop("tables", UNSET)
        for tables_item_data in _tables or []:
            tables_item = AttenuatedDDL.from_dict(tables_item_data)

            tables.append(tables_item)

        results_set = []
        _results_set = d.pop("results_set", UNSET)
        for results_set_item_data in _results_set or []:
            results_set_item = AskDispatchResponseResultsSetItem.from_dict(results_set_item_data)

            results_set.append(results_set_item)

        sql_text = d.pop("sql_text", UNSET)

        row_count = d.pop("row_count", UNSET)

        ask_dispatch_response = cls(
            id=id,
            query_text=query_text,
            status=status,
            tables=tables,
            results_set=results_set,
            sql_text=sql_text,
            row_count=row_count,
        )

        ask_dispatch_response.additional_properties = d
        return ask_dispatch_response

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import TYPE_CHECKING, Any, Dict, List, Type, TypeVar, Union, cast

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.dubo_example import DuboExample


T = TypeVar("T", bound="BodyReadQueryV1DuboQueryGet")


@_attrs_define
class BodyReadQueryV1DuboQueryGet:
    """
    Attributes:
        query_examples (Union[Unset, List['DuboExample']]): The query examples to use
        data_sample (Union[Unset, List[List[Any]]]): The data sample to use
    """

    query_examples: Union[Unset, List["DuboExample"]] = UNSET
    data_sample: Union[Unset, List[List[Any]]] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        query_examples: Union[Unset, List[Dict[str, Any]]] = UNSET
        if not isinstance(self.query_examples, Unset):
            query_examples = []
            for query_examples_item_data in self.query_examples:
                query_examples_item = query_examples_item_data.to_dict()

                query_examples.append(query_examples_item)

        data_sample: Union[Unset, List[List[Any]]] = UNSET
        if not isinstance(self.data_sample, Unset):
            data_sample = []
            for data_sample_item_data in self.data_sample:
                data_sample_item = data_sample_item_data

                data_sample.append(data_sample_item)

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if query_examples is not UNSET:
            field_dict["query_examples"] = query_examples
        if data_sample is not UNSET:
            field_dict["data_sample"] = data_sample

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.dubo_example import DuboExample

        d = src_dict.copy()
        query_examples = []
        _query_examples = d.pop("query_examples", UNSET)
        for query_examples_item_data in _query_examples or []:
            query_examples_item = DuboExample.from_dict(query_examples_item_data)

            query_examples.append(query_examples_item)

        data_sample = []
        _data_sample = d.pop("data_sample", UNSET)
        for data_sample_item_data in _data_sample or []:
            data_sample_item = cast(List[Any], data_sample_item_data)

            data_sample.append(data_sample_item)

        body_read_query_v1_dubo_query_get = cls(
            query_examples=query_examples,
            data_sample=data_sample,
        )

        body_read_query_v1_dubo_query_get.additional_properties = d
        return body_read_query_v1_dubo_query_get

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



import time
from IPython.display import clear_output, display


class DuboException(Exception):
    def __init__(self, msg: str, *args: object) -> None:
        super().__init__(*args)

        self.msg = msg

    def __str__(self) -> str:
        return self.msg


def in_jupyter():
    try:
        from IPython.display import clear_output  # noqa

        return True
    except ImportError:
        return False


def loading_icon(duration=5):
    if not in_jupyter():
        return
    symbols = ["*---", "-*--", "--*-", "---*", "--*-", "-*--"]
    end_time = time.time() + duration

    while time.time() < end_time:
        for symbol in symbols:
            clear_output(wait=True)  # noqa
            display(symbol)
            time.sleep(0.2)



from http import HTTPStatus
from typing import Any, Dict, Optional, Union

import httpx

from ... import errors
from ...client import AuthenticatedClient, Client
from ...models.http_validation_error import HTTPValidationError
from ...models.page_matched_doc import PageMatchedDoc
from ...types import UNSET, Response, Unset


def _get_kwargs(
    *,
    user_query: str,
    data_source_documentation_id: Union[Unset, str] = UNSET,
    page_number: Union[Unset, int] = 1,
    page_size: Union[Unset, int] = 25,
    x_dubo_key: str,
) -> Dict[str, Any]:
    headers = {}
    headers["x-dubo-key"] = x_dubo_key

    params: Dict[str, Any] = {}
    params["user_query"] = user_query

    params["data_source_documentation_id"] = data_source_documentation_id

    params["page_number"] = page_number

    params["page_size"] = page_size

    params = {k: v for k, v in params.items() if v is not UNSET and v is not None}

    return {
        "method": "get",
        "url": "/api/v1/dubo/query/filter-documentation",
        "params": params,
        "headers": headers,
    }


def _parse_response(
    *, client: Union[AuthenticatedClient, Client], response: httpx.Response
) -> Optional[Union[HTTPValidationError, PageMatchedDoc]]:
    if response.status_code == HTTPStatus.OK:
        response_200 = PageMatchedDoc.from_dict(response.json())

        return response_200
    if response.status_code == HTTPStatus.UNPROCESSABLE_ENTITY:
        response_422 = HTTPValidationError.from_dict(response.json())

        return response_422
    if client.raise_on_unexpected_status:
        raise errors.UnexpectedStatus(response.status_code, response.content)
    else:
        return None


def _build_response(
    *, client: Union[AuthenticatedClient, Client], response: httpx.Response
) -> Response[Union[HTTPValidationError, PageMatchedDoc]]:
    return Response(
        status_code=HTTPStatus(response.status_code),
        content=response.content,
        headers=response.headers,
        parsed=_parse_response(client=client, response=response),
    )


def sync_detailed(
    *,
    client: Union[AuthenticatedClient, Client],
    user_query: str,
    data_source_documentation_id: Union[Unset, str] = UNSET,
    page_number: Union[Unset, int] = 1,
    page_size: Union[Unset, int] = 25,
    x_dubo_key: str,
) -> Response[Union[HTTPValidationError, PageMatchedDoc]]:
    """Filter Documentation Endpoint

    Args:
        user_query (str):
        data_source_documentation_id (Union[Unset, str]):
        page_number (Union[Unset, int]): Page number Default: 1.
        page_size (Union[Unset, int]): Page size Default: 25.
        x_dubo_key (str):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[Union[HTTPValidationError, PageMatchedDoc]]
    """

    kwargs = _get_kwargs(
        user_query=user_query,
        data_source_documentation_id=data_source_documentation_id,
        page_number=page_number,
        page_size=page_size,
        x_dubo_key=x_dubo_key,
    )

    response = client.get_httpx_client().request(
        **kwargs,
    )

    return _build_response(client=client, response=response)


def sync(
    *,
    client: Union[AuthenticatedClient, Client],
    user_query: str,
    data_source_documentation_id: Union[Unset, str] = UNSET,
    page_number: Union[Unset, int] = 1,
    page_size: Union[Unset, int] = 25,
    x_dubo_key: str,
) -> Optional[Union[HTTPValidationError, PageMatchedDoc]]:
    """Filter Documentation Endpoint

    Args:
        user_query (str):
        data_source_documentation_id (Union[Unset, str]):
        page_number (Union[Unset, int]): Page number Default: 1.
        page_size (Union[Unset, int]): Page size Default: 25.
        x_dubo_key (str):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Union[HTTPValidationError, PageMatchedDoc]
    """

    return sync_detailed(
        client=client,
        user_query=user_query,
        data_source_documentation_id=data_source_documentation_id,
        page_number=page_number,
        page_size=page_size,
        x_dubo_key=x_dubo_key,
    ).parsed


async def asyncio_detailed(
    *,
    client: Union[AuthenticatedClient, Client],
    user_query: str,
    data_source_documentation_id: Union[Unset, str] = UNSET,
    page_number: Union[Unset, int] = 1,
    page_size: Union[Unset, int] = 25,
    x_dubo_key: str,
) -> Response[Union[HTTPValidationError, PageMatchedDoc]]:
    """Filter Documentation Endpoint

    Args:
        user_query (str):
        data_source_documentation_id (Union[Unset, str]):
        page_number (Union[Unset, int]): Page number Default: 1.
        page_size (Union[Unset, int]): Page size Default: 25.
        x_dubo_key (str):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[Union[HTTPValidationError, PageMatchedDoc]]
    """

    kwargs = _get_kwargs(
        user_query=user_query,
        data_source_documentation_id=data_source_documentation_id,
        page_number=page_number,
        page_size=page_size,
        x_dubo_key=x_dubo_key,
    )

    response = await client.get_async_httpx_client().request(**kwargs)

    return _build_response(client=client, response=response)


async def asyncio(
    *,
    client: Union[AuthenticatedClient, Client],
    user_query: str,
    data_source_documentation_id: Union[Unset, str] = UNSET,
    page_number: Union[Unset, int] = 1,
    page_size: Union[Unset, int] = 25,
    x_dubo_key: str,
) -> Optional[Union[HTTPValidationError, PageMatchedDoc]]:
    """Filter Documentation Endpoint

    Args:
        user_query (str):
        data_source_documentation_id (Union[Unset, str]):
        page_number (Union[Unset, int]): Page number Default: 1.
        page_size (Union[Unset, int]): Page size Default: 25.
        x_dubo_key (str):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Union[HTTPValidationError, PageMatchedDoc]
    """

    return (
        await asyncio_detailed(
            client=client,
            user_query=user_query,
            data_source_documentation_id=data_source_documentation_id,
            page_number=page_number,
            page_size=page_size,
            x_dubo_key=x_dubo_key,
        )
    ).parsed



"""Check if the config file exists, if not, create it."""

from dotenv import load_dotenv
import os

from dubo.common import DuboException

load_dotenv()
BASE_API_URL = os.getenv("DUBO_BASE_URL") or "https://api.dubo.gg"
RETRIEVE_RESULT_MAX_DELAY = float(os.getenv("DUBO_RETRIEVE_RESULT_MAX_DELAY", "10"))

DOCS_URL = "https://docs.dubo.gg/"
FEEDBACK_URL = "founders@dubo.gg"

_api_key = os.environ.get("DUBO_API_KEY", None)


def set_dubo_key(key: str):
    """
    Set the API key.
    """
    global _api_key
    _api_key = key


def get_dubo_key():
    """
    Get the API key.
    """
    if not _api_key:
        raise DuboException(
            "You must set the DUBO_API_KEY environment variable to use "
            "this function."
        )
    return _api_key



from http import HTTPStatus
from typing import Any, Dict, List, Optional, Union, cast

import httpx

from ... import errors
from ...client import AuthenticatedClient, Client
from ...models.body_read_query_v1_dubo_query_get import BodyReadQueryV1DuboQueryGet
from ...models.query_response import QueryResponse
from ...types import UNSET, Response, Unset


def _get_kwargs(
    *,
    json_body: BodyReadQueryV1DuboQueryGet,
    query: Union[Unset, str] = UNSET,
    user_query: Union[Unset, str] = UNSET,
    schemas: Union[Unset, List[str]] = UNSET,
    descriptions: Union[Unset, List[str]] = UNSET,
    data_header: Union[Unset, List[str]] = UNSET,
    macros: Union[Unset, bool] = False,
    fast: Union[Unset, bool] = False,
    model: Union[Unset, str] = UNSET,
) -> Dict[str, Any]:
    params: Dict[str, Any] = {}
    params["query"] = query

    params["user_query"] = user_query

    json_schemas: Union[Unset, List[str]] = UNSET
    if not isinstance(schemas, Unset):
        json_schemas = schemas

    params["schemas"] = json_schemas

    json_descriptions: Union[Unset, List[str]] = UNSET
    if not isinstance(descriptions, Unset):
        json_descriptions = descriptions

    params["descriptions"] = json_descriptions

    json_data_header: Union[Unset, List[str]] = UNSET
    if not isinstance(data_header, Unset):
        json_data_header = data_header

    params["data_header"] = json_data_header

    params["macros"] = macros

    params["fast"] = fast

    params["model"] = model

    params = {k: v for k, v in params.items() if v is not UNSET and v is not None}

    json_json_body = json_body.to_dict()

    return {
        "method": "get",
        "url": "/v1/dubo/query",
        "json": json_json_body,
        "params": params,
    }


def _parse_response(
    *, client: Union[AuthenticatedClient, Client], response: httpx.Response
) -> Optional[Union[Any, QueryResponse]]:
    if response.status_code == HTTPStatus.OK:
        response_200 = QueryResponse.from_dict(response.json())

        return response_200
    if response.status_code == HTTPStatus.UNPROCESSABLE_ENTITY:
        response_422 = cast(Any, None)
        return response_422
    if response.status_code == HTTPStatus.TOO_MANY_REQUESTS:
        response_429 = cast(Any, None)
        return response_429
    if response.status_code == HTTPStatus.INTERNAL_SERVER_ERROR:
        response_500 = cast(Any, None)
        return response_500
    if response.status_code == HTTPStatus.SERVICE_UNAVAILABLE:
        response_503 = cast(Any, None)
        return response_503
    if client.raise_on_unexpected_status:
        raise errors.UnexpectedStatus(response.status_code, response.content)
    else:
        return None


def _build_response(
    *, client: Union[AuthenticatedClient, Client], response: httpx.Response
) -> Response[Union[Any, QueryResponse]]:
    return Response(
        status_code=HTTPStatus(response.status_code),
        content=response.content,
        headers=response.headers,
        parsed=_parse_response(client=client, response=response),
    )


def sync_detailed(
    *,
    client: Union[AuthenticatedClient, Client],
    json_body: BodyReadQueryV1DuboQueryGet,
    query: Union[Unset, str] = UNSET,
    user_query: Union[Unset, str] = UNSET,
    schemas: Union[Unset, List[str]] = UNSET,
    descriptions: Union[Unset, List[str]] = UNSET,
    data_header: Union[Unset, List[str]] = UNSET,
    macros: Union[Unset, bool] = False,
    fast: Union[Unset, bool] = False,
    model: Union[Unset, str] = UNSET,
) -> Response[Union[Any, QueryResponse]]:
    """Convert text to SQL

    Args:
        query (Union[Unset, str]): The question to answer
        user_query (Union[Unset, str]): The question to answer
        schemas (Union[Unset, List[str]]): The table schema(s) to use
        descriptions (Union[Unset, List[str]]): The table description(s) to use
        data_header (Union[Unset, List[str]]): The data header to use
        macros (Union[Unset, bool]): Enable or disable macros
        fast (Union[Unset, bool]): Use faster less accurate model
        model (Union[Unset, str]): Model to use
        json_body (BodyReadQueryV1DuboQueryGet):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[Union[Any, QueryResponse]]
    """

    kwargs = _get_kwargs(
        json_body=json_body,
        query=query,
        user_query=user_query,
        schemas=schemas,
        descriptions=descriptions,
        data_header=data_header,
        macros=macros,
        fast=fast,
        model=model,
    )

    response = client.get_httpx_client().request(
        **kwargs,
    )

    return _build_response(client=client, response=response)


def sync(
    *,
    client: Union[AuthenticatedClient, Client],
    json_body: BodyReadQueryV1DuboQueryGet,
    query: Union[Unset, str] = UNSET,
    user_query: Union[Unset, str] = UNSET,
    schemas: Union[Unset, List[str]] = UNSET,
    descriptions: Union[Unset, List[str]] = UNSET,
    data_header: Union[Unset, List[str]] = UNSET,
    macros: Union[Unset, bool] = False,
    fast: Union[Unset, bool] = False,
    model: Union[Unset, str] = UNSET,
) -> Optional[Union[Any, QueryResponse]]:
    """Convert text to SQL

    Args:
        query (Union[Unset, str]): The question to answer
        user_query (Union[Unset, str]): The question to answer
        schemas (Union[Unset, List[str]]): The table schema(s) to use
        descriptions (Union[Unset, List[str]]): The table description(s) to use
        data_header (Union[Unset, List[str]]): The data header to use
        macros (Union[Unset, bool]): Enable or disable macros
        fast (Union[Unset, bool]): Use faster less accurate model
        model (Union[Unset, str]): Model to use
        json_body (BodyReadQueryV1DuboQueryGet):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Union[Any, QueryResponse]
    """

    return sync_detailed(
        client=client,
        json_body=json_body,
        query=query,
        user_query=user_query,
        schemas=schemas,
        descriptions=descriptions,
        data_header=data_header,
        macros=macros,
        fast=fast,
        model=model,
    ).parsed


async def asyncio_detailed(
    *,
    client: Union[AuthenticatedClient, Client],
    json_body: BodyReadQueryV1DuboQueryGet,
    query: Union[Unset, str] = UNSET,
    user_query: Union[Unset, str] = UNSET,
    schemas: Union[Unset, List[str]] = UNSET,
    descriptions: Union[Unset, List[str]] = UNSET,
    data_header: Union[Unset, List[str]] = UNSET,
    macros: Union[Unset, bool] = False,
    fast: Union[Unset, bool] = False,
    model: Union[Unset, str] = UNSET,
) -> Response[Union[Any, QueryResponse]]:
    """Convert text to SQL

    Args:
        query (Union[Unset, str]): The question to answer
        user_query (Union[Unset, str]): The question to answer
        schemas (Union[Unset, List[str]]): The table schema(s) to use
        descriptions (Union[Unset, List[str]]): The table description(s) to use
        data_header (Union[Unset, List[str]]): The data header to use
        macros (Union[Unset, bool]): Enable or disable macros
        fast (Union[Unset, bool]): Use faster less accurate model
        model (Union[Unset, str]): Model to use
        json_body (BodyReadQueryV1DuboQueryGet):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[Union[Any, QueryResponse]]
    """

    kwargs = _get_kwargs(
        json_body=json_body,
        query=query,
        user_query=user_query,
        schemas=schemas,
        descriptions=descriptions,
        data_header=data_header,
        macros=macros,
        fast=fast,
        model=model,
    )

    response = await client.get_async_httpx_client().request(**kwargs)

    return _build_response(client=client, response=response)


async def asyncio(
    *,
    client: Union[AuthenticatedClient, Client],
    json_body: BodyReadQueryV1DuboQueryGet,
    query: Union[Unset, str] = UNSET,
    user_query: Union[Unset, str] = UNSET,
    schemas: Union[Unset, List[str]] = UNSET,
    descriptions: Union[Unset, List[str]] = UNSET,
    data_header: Union[Unset, List[str]] = UNSET,
    macros: Union[Unset, bool] = False,
    fast: Union[Unset, bool] = False,
    model: Union[Unset, str] = UNSET,
) -> Optional[Union[Any, QueryResponse]]:
    """Convert text to SQL

    Args:
        query (Union[Unset, str]): The question to answer
        user_query (Union[Unset, str]): The question to answer
        schemas (Union[Unset, List[str]]): The table schema(s) to use
        descriptions (Union[Unset, List[str]]): The table description(s) to use
        data_header (Union[Unset, List[str]]): The data header to use
        macros (Union[Unset, bool]): Enable or disable macros
        fast (Union[Unset, bool]): Use faster less accurate model
        model (Union[Unset, str]): Model to use
        json_body (BodyReadQueryV1DuboQueryGet):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Union[Any, QueryResponse]
    """

    return (
        await asyncio_detailed(
            client=client,
            json_body=json_body,
            query=query,
            user_query=user_query,
            schemas=schemas,
            descriptions=descriptions,
            data_header=data_header,
            macros=macros,
            fast=fast,
            model=model,
        )
    ).parsed



from typing import Any, Dict, List, Type, TypeVar, Union

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

T = TypeVar("T", bound="MatchedDoc")


@_attrs_define
class MatchedDoc:
    """
    Attributes:
        body (str):
        score (float):
        matched_doc_id (Union[Unset, str]):
    """

    body: str
    score: float
    matched_doc_id: Union[Unset, str] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        body = self.body
        score = self.score
        matched_doc_id = self.matched_doc_id

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "body": body,
                "score": score,
            }
        )
        if matched_doc_id is not UNSET:
            field_dict["matched_doc_id"] = matched_doc_id

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        body = d.pop("body")

        score = d.pop("score")

        matched_doc_id = d.pop("matched_doc_id", UNSET)

        matched_doc = cls(
            body=body,
            score=score,
            matched_doc_id=matched_doc_id,
        )

        matched_doc.additional_properties = d
        return matched_doc

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from io import BytesIO
from typing import Any, Dict, List, Type, TypeVar

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import File

T = TypeVar("T", bound="BodyUpdateDocumentApiV1DuboDocumentationPut")


@_attrs_define
class BodyUpdateDocumentApiV1DuboDocumentationPut:
    """
    Attributes:
        file (File):
    """

    file: File
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        file = self.file.to_tuple()

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "file": file,
            }
        )

        return field_dict

    def to_multipart(self) -> Dict[str, Any]:
        file = self.file.to_tuple()

        field_dict: Dict[str, Any] = {}
        field_dict.update(
            {key: (None, str(value).encode(), "text/plain") for key, value in self.additional_properties.items()}
        )
        field_dict.update(
            {
                "file": file,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        file = File(payload=BytesIO(d.pop("file")))

        body_update_document_api_v1_dubo_documentation_put = cls(
            file=file,
        )

        body_update_document_api_v1_dubo_documentation_put.additional_properties = d
        return body_update_document_api_v1_dubo_documentation_put

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from http import HTTPStatus
from typing import Any, Dict, Optional, Union, cast

import httpx

from ... import errors
from ...client import AuthenticatedClient, Client
from ...models.body_update_document_api_v1_dubo_documentation_put import BodyUpdateDocumentApiV1DuboDocumentationPut
from ...models.http_validation_error import HTTPValidationError
from ...types import UNSET, Response, Unset


def _get_kwargs(
    *,
    multipart_data: BodyUpdateDocumentApiV1DuboDocumentationPut,
    data_source_documentation_id: str,
    shingle_length: Union[Unset, int] = 1000,
    step: Union[Unset, int] = 500,
    x_dubo_key: str,
) -> Dict[str, Any]:
    headers = {}
    headers["x-dubo-key"] = x_dubo_key

    params: Dict[str, Any] = {}
    params["data_source_documentation_id"] = data_source_documentation_id

    params["shingle_length"] = shingle_length

    params["step"] = step

    params = {k: v for k, v in params.items() if v is not UNSET and v is not None}

    multipart_multipart_data = multipart_data.to_multipart()

    return {
        "method": "put",
        "url": "/api/v1/dubo/documentation",
        "files": multipart_multipart_data,
        "params": params,
        "headers": headers,
    }


def _parse_response(
    *, client: Union[AuthenticatedClient, Client], response: httpx.Response
) -> Optional[Union[HTTPValidationError, bool]]:
    if response.status_code == HTTPStatus.OK:
        response_200 = cast(bool, response.json())
        return response_200
    if response.status_code == HTTPStatus.UNPROCESSABLE_ENTITY:
        response_422 = HTTPValidationError.from_dict(response.json())

        return response_422
    if client.raise_on_unexpected_status:
        raise errors.UnexpectedStatus(response.status_code, response.content)
    else:
        return None


def _build_response(
    *, client: Union[AuthenticatedClient, Client], response: httpx.Response
) -> Response[Union[HTTPValidationError, bool]]:
    return Response(
        status_code=HTTPStatus(response.status_code),
        content=response.content,
        headers=response.headers,
        parsed=_parse_response(client=client, response=response),
    )


def sync_detailed(
    *,
    client: Union[AuthenticatedClient, Client],
    multipart_data: BodyUpdateDocumentApiV1DuboDocumentationPut,
    data_source_documentation_id: str,
    shingle_length: Union[Unset, int] = 1000,
    step: Union[Unset, int] = 500,
    x_dubo_key: str,
) -> Response[Union[HTTPValidationError, bool]]:
    """Update Document

    Args:
        data_source_documentation_id (str):
        shingle_length (Union[Unset, int]):  Default: 1000.
        step (Union[Unset, int]):  Default: 500.
        x_dubo_key (str):
        multipart_data (BodyUpdateDocumentApiV1DuboDocumentationPut):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[Union[HTTPValidationError, bool]]
    """

    kwargs = _get_kwargs(
        multipart_data=multipart_data,
        data_source_documentation_id=data_source_documentation_id,
        shingle_length=shingle_length,
        step=step,
        x_dubo_key=x_dubo_key,
    )

    response = client.get_httpx_client().request(
        **kwargs,
    )

    return _build_response(client=client, response=response)


def sync(
    *,
    client: Union[AuthenticatedClient, Client],
    multipart_data: BodyUpdateDocumentApiV1DuboDocumentationPut,
    data_source_documentation_id: str,
    shingle_length: Union[Unset, int] = 1000,
    step: Union[Unset, int] = 500,
    x_dubo_key: str,
) -> Optional[Union[HTTPValidationError, bool]]:
    """Update Document

    Args:
        data_source_documentation_id (str):
        shingle_length (Union[Unset, int]):  Default: 1000.
        step (Union[Unset, int]):  Default: 500.
        x_dubo_key (str):
        multipart_data (BodyUpdateDocumentApiV1DuboDocumentationPut):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Union[HTTPValidationError, bool]
    """

    return sync_detailed(
        client=client,
        multipart_data=multipart_data,
        data_source_documentation_id=data_source_documentation_id,
        shingle_length=shingle_length,
        step=step,
        x_dubo_key=x_dubo_key,
    ).parsed


async def asyncio_detailed(
    *,
    client: Union[AuthenticatedClient, Client],
    multipart_data: BodyUpdateDocumentApiV1DuboDocumentationPut,
    data_source_documentation_id: str,
    shingle_length: Union[Unset, int] = 1000,
    step: Union[Unset, int] = 500,
    x_dubo_key: str,
) -> Response[Union[HTTPValidationError, bool]]:
    """Update Document

    Args:
        data_source_documentation_id (str):
        shingle_length (Union[Unset, int]):  Default: 1000.
        step (Union[Unset, int]):  Default: 500.
        x_dubo_key (str):
        multipart_data (BodyUpdateDocumentApiV1DuboDocumentationPut):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[Union[HTTPValidationError, bool]]
    """

    kwargs = _get_kwargs(
        multipart_data=multipart_data,
        data_source_documentation_id=data_source_documentation_id,
        shingle_length=shingle_length,
        step=step,
        x_dubo_key=x_dubo_key,
    )

    response = await client.get_async_httpx_client().request(**kwargs)

    return _build_response(client=client, response=response)


async def asyncio(
    *,
    client: Union[AuthenticatedClient, Client],
    multipart_data: BodyUpdateDocumentApiV1DuboDocumentationPut,
    data_source_documentation_id: str,
    shingle_length: Union[Unset, int] = 1000,
    step: Union[Unset, int] = 500,
    x_dubo_key: str,
) -> Optional[Union[HTTPValidationError, bool]]:
    """Update Document

    Args:
        data_source_documentation_id (str):
        shingle_length (Union[Unset, int]):  Default: 1000.
        step (Union[Unset, int]):  Default: 500.
        x_dubo_key (str):
        multipart_data (BodyUpdateDocumentApiV1DuboDocumentationPut):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Union[HTTPValidationError, bool]
    """

    return (
        await asyncio_detailed(
            client=client,
            multipart_data=multipart_data,
            data_source_documentation_id=data_source_documentation_id,
            shingle_length=shingle_length,
            step=step,
            x_dubo_key=x_dubo_key,
        )
    ).parsed



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['dubo',
 'dubo.api_client',
 'dubo.api_client.api',
 'dubo.api_client.api.dubo',
 'dubo.api_client.api.enterprise',
 'dubo.api_client.api.sdk',
 'dubo.api_client.models']

package_data = \
{'': ['*']}

install_requires = \
['altair>=5.0.1,<6.0.0',
 'attrs>=21.3.0',
 'httpx>=0.20.0,<0.25.0',
 'pandas>=1.0.0,<2.0.0',
 'pydeck>=0.8.0,<0.9.0',
 'python-dateutil>=2.8.0,<3.0.0',
 'python-dotenv>=1.0.0,<2.0.0']

setup_kwargs = {
    'name': 'dubo',
    'version': '0.2.8',
    'description': 'Analytics made simple',
    'long_description': 'dubo\n====\n\ndubo is a client for running LLMs against DataFrames and other 2D data.\n\nIt is currently in a beta release. To join the beta, you can check out the [documentation](https://docs.dubo.gg).\n\n## Generate API Client\n\nAn api client can be generated to access the dubo API, it is located in the `/dubo/api_client` folder:\n\n```shell\njust generate-api-client\n```\n\nMore info can be found in the README file inside the `api-client-generator` folder.\n\n## Generate Documentation\n\n```shell\njust generate-doc\n```\n',
    'author': 'Andrew Duberstein',
    'author_email': 'ajduberstein@gmail.com',
    'maintainer': 'None',
    'maintainer_email': 'None',
    'url': 'None',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.8,<4.0',
}


setup(**setup_kwargs)



from typing import TYPE_CHECKING, Any, Dict, List, Type, TypeVar, Union

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..models.chart_type import ChartType
from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.dubo_chart_query_chart_spec import DuboChartQueryChartSpec
    from ..models.dubo_chart_query_data_snippet_item import DuboChartQueryDataSnippetItem


T = TypeVar("T", bound="DuboChartQuery")


@_attrs_define
class DuboChartQuery:
    """
    Attributes:
        user_query (str):
        data_snippet (List['DuboChartQueryDataSnippetItem']):
        fast (Union[Unset, bool]): Use faster less accurate model
        chart_spec (Union[Unset, DuboChartQueryChartSpec]):
        chart_type (Union[Unset, ChartType]): An enumeration. Default: ChartType.VEGA_LITE.
        thread_id (Union[Unset, str]):
    """

    user_query: str
    data_snippet: List["DuboChartQueryDataSnippetItem"]
    fast: Union[Unset, bool] = False
    chart_spec: Union[Unset, "DuboChartQueryChartSpec"] = UNSET
    chart_type: Union[Unset, ChartType] = ChartType.VEGA_LITE
    thread_id: Union[Unset, str] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        user_query = self.user_query
        data_snippet = []
        for data_snippet_item_data in self.data_snippet:
            data_snippet_item = data_snippet_item_data.to_dict()

            data_snippet.append(data_snippet_item)

        fast = self.fast
        chart_spec: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.chart_spec, Unset):
            chart_spec = self.chart_spec.to_dict()

        chart_type: Union[Unset, str] = UNSET
        if not isinstance(self.chart_type, Unset):
            chart_type = self.chart_type.value

        thread_id = self.thread_id

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "user_query": user_query,
                "data_snippet": data_snippet,
            }
        )
        if fast is not UNSET:
            field_dict["fast"] = fast
        if chart_spec is not UNSET:
            field_dict["chart_spec"] = chart_spec
        if chart_type is not UNSET:
            field_dict["chart_type"] = chart_type
        if thread_id is not UNSET:
            field_dict["thread_id"] = thread_id

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.dubo_chart_query_chart_spec import DuboChartQueryChartSpec
        from ..models.dubo_chart_query_data_snippet_item import DuboChartQueryDataSnippetItem

        d = src_dict.copy()
        user_query = d.pop("user_query")

        data_snippet = []
        _data_snippet = d.pop("data_snippet")
        for data_snippet_item_data in _data_snippet:
            data_snippet_item = DuboChartQueryDataSnippetItem.from_dict(data_snippet_item_data)

            data_snippet.append(data_snippet_item)

        fast = d.pop("fast", UNSET)

        _chart_spec = d.pop("chart_spec", UNSET)
        chart_spec: Union[Unset, DuboChartQueryChartSpec]
        if isinstance(_chart_spec, Unset):
            chart_spec = UNSET
        else:
            chart_spec = DuboChartQueryChartSpec.from_dict(_chart_spec)

        _chart_type = d.pop("chart_type", UNSET)
        chart_type: Union[Unset, ChartType]
        if isinstance(_chart_type, Unset):
            chart_type = UNSET
        else:
            chart_type = ChartType(_chart_type)

        thread_id = d.pop("thread_id", UNSET)

        dubo_chart_query = cls(
            user_query=user_query,
            data_snippet=data_snippet,
            fast=fast,
            chart_spec=chart_spec,
            chart_type=chart_type,
            thread_id=thread_id,
        )

        dubo_chart_query.additional_properties = d
        return dubo_chart_query

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties
